@@ -1,47 +1,58 @@
 /*
- * Asterisk -- A telephony toolkit for Linux.
+ * Asterisk -- An open source telephony toolkit.
  *
- * Call Detail Record API 
- * 
- * Copyright (C) 1999, Mark Spencer
+ * Copyright (C) 1999 - 2005, Digium, Inc.
  *
- * Mark Spencer <markster@linux-support.net>
+ * Mark Spencer <markster@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
  *
  * This program is free software, distributed under the terms of
- * the GNU General Public License.
- *
- * Includes code and algorithms from the Zapata library.
- *
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
  */
 
-#ifndef _CDR_H
-#define _CDR_H
+/*! \file
+ * \brief Call Detail Record API 
+ */
+
+#ifndef _ASTERISK_CDR_H
+#define _ASTERISK_CDR_H
 
-#include <asterisk/channel.h>
 #include <sys/time.h>
-
+#define AST_CDR_FLAG_KEEP_VARS			(1 << 0)
 #define AST_CDR_FLAG_POSTED			(1 << 1)
 #define AST_CDR_FLAG_LOCKED			(1 << 2)
 #define AST_CDR_FLAG_CHILD			(1 << 3)
+#define AST_CDR_FLAG_POST_DISABLED		(1 << 4)
 
 #define AST_CDR_NOANSWER			(1 << 0)
 #define AST_CDR_BUSY				(1 << 1)
 #define AST_CDR_ANSWERED			(1 << 2)
 #define AST_CDR_FAILED				(1 << 3)
 
-//! AMA Flags
+/*! AMA Flags */
 #define AST_CDR_OMIT				(1)
 #define AST_CDR_BILLING				(2)
 #define AST_CDR_DOCUMENTATION			(3)
 
 #define AST_MAX_USER_FIELD			256
+#define AST_MAX_ACCOUNT_CODE			20
+
+/* Include channel.h after relevant declarations it will need */
+#include "asterisk/channel.h"
+#include "asterisk/utils.h"
 
 struct ast_channel;
 
-//! Responsible for call detail data
+/*! Responsible for call detail data */
 struct ast_cdr {
 	/*! Caller*ID with text */
-	char clid[AST_MAX_EXTENSION];		
+	char clid[AST_MAX_EXTENSION];
 	/*! Caller*ID number */
 	char src[AST_MAX_EXTENSION];		
 	/*! Destination extension */
@@ -71,32 +82,45 @@ struct ast_cdr {
 	/*! What flags to use */
 	int amaflags;				
 	/*! What account number to use */
-	char accountcode[20];			
+	char accountcode[AST_MAX_ACCOUNT_CODE];			
 	/*! flags */
-	int flags;				
+	unsigned int flags;				
 	/* Unique Channel Identifier */
 	char uniqueid[32];
 	/* User field */
 	char userfield[AST_MAX_USER_FIELD];
+
+	/* A linked list for variables */
+	struct varshead varshead;
+
 	struct ast_cdr *next;
 };
 
+extern void ast_cdr_getvar(struct ast_cdr *cdr, const char *name, char **ret, char *workspace, int workspacelen, int recur);
+extern int ast_cdr_setvar(struct ast_cdr *cdr, const char *name, const char *value, int recur);
+extern int ast_cdr_serialize_variables(struct ast_cdr *cdr, char *buf, size_t size, char delim, char sep, int recur);
+extern void ast_cdr_free_vars(struct ast_cdr *cdr, int recur);
+extern int ast_cdr_copy_vars(struct ast_cdr *to_cdr, struct ast_cdr *from_cdr);
+
 typedef int (*ast_cdrbe)(struct ast_cdr *cdr);
 
-//! Allocate a record
-/*! 
+/*! \brief Allocate a CDR record 
  * Returns a malloc'd ast_cdr structure, returns NULL on error (malloc failure)
  */
 extern struct ast_cdr *ast_cdr_alloc(void);
 
-//! Free a record
-/* \param cdr ast_cdr structure to free
+/*! \brief Duplicate a record 
+ * Returns a malloc'd ast_cdr structure, returns NULL on error (malloc failure)
+ */
+extern struct ast_cdr *ast_cdr_dup(struct ast_cdr *cdr);
+
+/*! \brief Free a CDR record 
+ * \param cdr ast_cdr structure to free
  * Returns nothing important
  */
 extern void ast_cdr_free(struct ast_cdr *cdr);
 
-//! Initialize based on a channel
-/*! 
+/*! \brief Initialize based on a channel
  * \param cdr Call Detail Record to use for channel
  * \param chan Channel to bind CDR with
  * Initializes a CDR and associates it with a particular channel
@@ -104,7 +128,7 @@ extern void ast_cdr_free(struct ast_cdr *cdr);
  */
 extern int ast_cdr_init(struct ast_cdr *cdr, struct ast_channel *chan);
 
-//! Initialize based on a channel
+/*! Initialize based on a channel */
 /*! 
  * \param cdr Call Detail Record to use for channel
  * \param chan Channel to bind CDR with
@@ -113,7 +137,7 @@ extern int ast_cdr_init(struct ast_cdr *cdr, struct ast_channel *chan);
  */
 extern int ast_cdr_setcid(struct ast_cdr *cdr, struct ast_channel *chan);
 
-//! Register a CDR handling engine
+/*! Register a CDR handling engine */
 /*!
  * \param name name associated with the particular CDR handler
  * \param desc description of the CDR handler
@@ -123,14 +147,14 @@ extern int ast_cdr_setcid(struct ast_cdr *cdr, struct ast_channel *chan);
  */
 extern int ast_cdr_register(char *name, char *desc, ast_cdrbe be);
 
-//! Unregister a CDR handling engine
+/*! Unregister a CDR handling engine */
 /*!
  * \param name name of CDR handler to unregister
  * Unregisters a CDR by it's name
  */
 extern void ast_cdr_unregister(char *name);
 
-//! Start a call
+/*! Start a call */
 /*!
  * \param cdr the cdr you wish to associate with the call
  * Starts all CDR stuff necessary for monitoring a call
@@ -138,28 +162,28 @@ extern void ast_cdr_unregister(char *name);
  */
 extern void ast_cdr_start(struct ast_cdr *cdr);
 
-//! Answer a call
+/*! Answer a call */
 /*!
  * \param cdr the cdr you wish to associate with the call
  * Starts all CDR stuff necessary for doing CDR when answering a call
  */
 extern void ast_cdr_answer(struct ast_cdr *cdr);
 
-//! Busy a call
+/*! Busy a call */
 /*!
  * \param cdr the cdr you wish to associate with the call
  * Returns nothing important
  */
 extern void ast_cdr_busy(struct ast_cdr *cdr);
 
-//! Fail a call
+/*! Fail a call */
 /*!
  * \param cdr the cdr you wish to associate with the call
  * Returns nothing important
  */
 extern void ast_cdr_failed(struct ast_cdr *cdr);
 
-//! Save the result of the call based on the AST_CAUSE_*
+/*! Save the result of the call based on the AST_CAUSE_* */
 /*!
  * \param cdr the cdr you wish to associate with the call
  * Returns nothing important
@@ -167,7 +191,7 @@ extern void ast_cdr_failed(struct ast_cdr *cdr);
  */
 extern int ast_cdr_disposition(struct ast_cdr *cdr, int cause);
 	
-//! End a call
+/*! End a call */
 /*!
  * \param cdr the cdr you have associated the call with
  * Registers the end of call time in the cdr structure.
@@ -175,23 +199,33 @@ extern int ast_cdr_disposition(struct ast_cdr *cdr, int cause);
  */
 extern void ast_cdr_end(struct ast_cdr *cdr);
 
-//! Post the detail record
+/*! Detaches the detail record for posting (and freeing) either now or at a
+ * later time in bulk with other records during batch mode operation */
 /*! 
- * \param cdr Which cdr to post
- * Actually outputs the CDR record to the CDR plugins installed
+ * \param cdr Which CDR to detach from the channel thread
+ * Prevents the channel thread from blocking on the CDR handling
  * Returns nothing
  */
-extern void ast_cdr_post(struct ast_cdr *cdr);
+extern void ast_cdr_detach(struct ast_cdr *cdr);
 
-//! Set the destination channel, if there was one
+/*! Spawns (possibly) a new thread to submit a batch of CDRs to the backend engines */
+/*!
+ * \param shutdown Whether or not we are shutting down
+ * Blocks the asterisk shutdown procedures until the CDR data is submitted.
+ * Returns nothing
+ */
+extern void ast_cdr_submit_batch(int shutdown);
+
+/*! Set the destination channel, if there was one */
 /*!
  * \param cdr Which cdr it's applied to
+ * \param chan Channel to which dest will be
  * Sets the destination channel the CDR is applied to
  * Returns nothing
  */
 extern void ast_cdr_setdestchan(struct ast_cdr *cdr, char *chan);
 
-//! Set the last executed application
+/*! Set the last executed application */
 /*!
  * \param cdr which cdr to act upon
  * \param app the name of the app you wish to change it to
@@ -201,30 +235,31 @@ extern void ast_cdr_setdestchan(struct ast_cdr *cdr, char *chan);
  */
 extern void ast_cdr_setapp(struct ast_cdr *cdr, char *app, char *data);
 
-//! Convert a string to a detail record AMA flag
+/*! Convert a string to a detail record AMA flag */
 /*!
  * \param flag string form of flag
  * Converts the string form of the flag to the binary form.
  * Returns the binary form of the flag
  */
-extern int ast_cdr_amaflags2int(char *flag);
+extern int ast_cdr_amaflags2int(const char *flag);
 
-//! Disposition to a string
+/*! Disposition to a string */
 /*!
- * \param flag input binary form
+ * \param disposition input binary form
  * Converts the binary form of a disposition to string form.
  * Returns a pointer to the string form
  */
 extern char *ast_cdr_disp2str(int disposition);
 
-//! Reset the detail record, optionally posting it first
+/*! Reset the detail record, optionally posting it first */
 /*!
  * \param cdr which cdr to act upon
- * \param post whether or not to post the cdr first before resetting it
+ * \param flags |AST_CDR_FLAG_POSTED whether or not to post the cdr first before resetting it
+ *              |AST_CDR_FLAG_LOCKED whether or not to reset locked CDR's
  */
-extern void ast_cdr_reset(struct ast_cdr *cdr, int post);
+extern void ast_cdr_reset(struct ast_cdr *cdr, struct ast_flags *flags);
 
-//! Flags to a string
+/*! Flags to a string */
 /*!
  * \param flags binary flag
  * Converts binary flags to string flags
@@ -232,12 +267,12 @@ extern void ast_cdr_reset(struct ast_cdr *cdr, int post);
  */
 extern char *ast_cdr_flags2str(int flags);
 
-extern int ast_cdr_setaccount(struct ast_channel *chan, char *account);
-extern int ast_cdr_setamaflags(struct ast_channel *chan, char *account);
+extern int ast_cdr_setaccount(struct ast_channel *chan, const char *account);
+extern int ast_cdr_setamaflags(struct ast_channel *chan, const char *amaflags);
 
 
-extern int ast_cdr_setuserfield(struct ast_channel *chan, char *userfield);
-extern int ast_cdr_appenduserfield(struct ast_channel *chan, char *userfield);
+extern int ast_cdr_setuserfield(struct ast_channel *chan, const char *userfield);
+extern int ast_cdr_appenduserfield(struct ast_channel *chan, const char *userfield);
 
 
 /* Update CDR on a channel */
@@ -246,12 +281,17 @@ extern int ast_cdr_update(struct ast_channel *chan);
 
 extern int ast_default_amaflags;
 
-extern char ast_default_accountcode[20];
+extern char ast_default_accountcode[AST_MAX_ACCOUNT_CODE];
 
-#define ast_cdr_has_flag(cdr, flag) ((cdr)->flags & (flag))
-#define ast_cdr_add_flag(cdr, flag) ((cdr)->flags |= (flag))
-#define ast_cdr_del_flag(cdr, flag) ((cdr)->flags &= ~(flag))
+extern struct ast_cdr *ast_cdr_append(struct ast_cdr *cdr, struct ast_cdr *newcdr);
 
-extern void ast_cdr_append(struct ast_cdr *cdr, struct ast_cdr *newcdr);
+/*! Reload the configuration file cdr.conf and start/stop CDR scheduling thread */
+extern void ast_cdr_engine_reload(void);
 
-#endif /* _CDR_H */
+/*! Load the configuration file cdr.conf and possibly start the CDR scheduling thread */
+extern int ast_cdr_engine_init(void);
+
+/*! Submit any remaining CDRs and prepare for shutdown */
+extern void ast_cdr_engine_term(void);
+
+#endif /* _ASTERISK_CDR_H */
