@@ -1,16 +1,23 @@
 /*
- * Asterisk -- A telephony toolkit for Linux.
+ * Asterisk -- An open source telephony toolkit.
  *
- * Asterisk architecture endianess compatibility definitions
- * 
  * Copyright (C) 1999 - 2005, Digium, Inc.
  *
  * Mark Spencer <markster@digium.com>
  *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
  * This program is free software, distributed under the terms of
- * the GNU Lesser General Public License.  Other components of
- * Asterisk are distributed under The GNU General Public License
- * only.
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ * \brief Asterisk architecture endianess compatibility definitions
  */
 
 #ifndef _ASTERISK_ENDIAN_H
@@ -20,9 +27,7 @@
  * Autodetect system endianess
  */
 
-#ifdef SOLARIS
-#include "solaris-compat/compat.h"
-#endif
+#include "asterisk/compat.h"
 
 #ifndef __BYTE_ORDER
 #ifdef __linux__
@@ -54,3 +59,4 @@
 #endif /* __BYTE_ORDER */
 
 #endif /* _ASTERISK_ENDIAN_H */
+
